#!/bin/sh

prefix=@bindir@
lockfile=/var/run/aexpl.pid
config=@confdir@/aexpl.conf

aexpl_start ()
  {
    kill -0 `cat $lockfile /dev/null 2>&1` > /dev/null 2>&1
    if [ $? == 0 ]; then
      echo "Anti Exploid daemon is already running."
      exit 1
    fi

    echo -n "Starting Anti Exploid daemon: "

    if [ -z "`/sbin/lsmod | grep "^dazuko "`" ]; then
      /sbin/modprobe dazuko > /dev/null 2>&1
     
      if [ $? != 0 ]; then
        echo "failed."
        exit 1
      fi
    fi

    if [ -x $prefix/aexpl ]; then
      $prefix/aexpl -c $config > /dev/null
    else
      echo "failed."
      exit 1
    fi

    if [ $? == 0 ]; then
      echo "done."
    else
      echo "failed."
    fi
  }

aexpl_stop ()
  {
    kill -0 `cat $lockfile /dev/null 2>&1` > /dev/null 2>&1
    if [ $? != 0 ]; then
      echo "Anti Exploid daemon not running."
      exit 1
    fi

    /sbin/rmmod dazuko > /dev/null 2>&1

    echo -n "Stopping Anti Exploid daemon: "

    kill `cat $lockfile /dev/null 2>&1` > /dev/null 2>&1
    if [ $? == 0 ]; then
      /sbin/rmmod dazuko > /dev/null 2>&1
      echo "done." 
    else
      echo "failed."
    fi
  }

aexpl_restart ()
  {
    aexpl_stop
    sleep 1
    aexpl_start
  }

aexpl_update ()
  {
    echo -n "Updating Anti Exploid database: "

    if [ -x $prefix/aexpl ]; then
      $prefix/aexpl -u $config > /dev/null
    else
      echo "failed."
      exit 1
    fi

    if [ $? == 0 ]; then
      echo "done."
    else
      echo "failed."
    fi
  }

case "$1" in
  'start')
    aexpl_start
    ;;
  'stop')
    aexpl_stop
    ;;
  'restart')
    aexpl_restart
    ;;
  'update')
    aexpl_update
    ;;
  *)
    echo "usage: $0 {start|stop|restart|update}"
esac
