/* log.c -- Log file handling
 *
 * Copyright (C) 2004 H Zero Seven
 *
 * Authors: Christian Volmering <risk@h07.org>
 *
 * This file is part of the aexpl-ng distribution.
 *
 * The aexpl-ng distribution is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The aexpl-ng distribution is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the aexpl-ng distribution; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include "config.h"

#include <time.h>
#include <sys/time.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>

#define SYSLOG_NAMES
#include <syslog.h>

#ifdef WITH_DMALLOC
#  include <dmalloc.h>
#endif

static FILE *log = NULL;

/* {{{ setup_log */
int
setup_log (const char *name, int facility, const char *file)
{
  openlog (name, LOG_PID | LOG_NDELAY, facility);

  if (file)
    {
      if (!(log = fopen (file, "a")))
	return (0);
    }

  return (1);
}

/* }}} */

/* {{{ teardown_log */
void
teardown_log (void)
{
  closelog ();
  if (log)
    fclose (log);
}

/* }}} */

/* {{{ write_log */
int
write_log (int priority, const char *format, ...)
{
  char buffer[128], log_str[256];
  va_list ap;
  int length, pos;
  struct timeval tv;
  char __ctime[26];
  struct tm tm;
  char pri_str[32];

  va_start (ap, format);
  vsnprintf (buffer, 128, format, ap);
  length = strlen (buffer);
  va_end (ap);

  syslog (priority, buffer);

  if (log)
    {
      if (gettimeofday (&tv, NULL) == -1)
	return (0);

      for (pos = 0; prioritynames[pos].c_name; pos++)
	if (prioritynames[pos].c_val == priority)
	  {
	    localtime_r ((time_t *) &tv.tv_sec, &tm);
	    strftime (__ctime, 26, "%a %b %e %T %Y", &tm);

	    snprintf (pri_str, 32, "<%s>", prioritynames[pos].c_name);

	    snprintf (log_str, 256, "%s %-7s %s\n", __ctime, pri_str, buffer);
	    break;
	  }

      if (fwrite (log_str, sizeof (char), strlen (log_str), log) < 0)
	return (0);

      fflush (log);
    }

  return (1);
}

/* }}} */

/*
 * Local variables:
 * tab-width: 8
 * c-basic-offset: 8
 * End:
 * vim600: noet nonu sw=8 ts=8 fdm=marker
 * vim<600: noet nonu sw=8 ts=8
 */
