/* md5sum.c -- Hash file contents with MD5 algorithm
 * 
 * Copyright (C) 2004 H Zero Seven
 *
 * Authors: Enrico Kern <phantom@h07.org>
 *          Christian Volmering <risk@h07.org>
 * 
 * This file is part of the aexpl-ng distribution.
 * 
 * The aexpl-ng distribution is free software; you can redistribute it 
 * and/or modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 of 
 * the License, or (at your option) any later version.
 *
 * The aexpl-ng distribution is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied 
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the aexpl-ng distribution; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 * MA 02111-1307 USA
 */

#include "config.h"
#include "md5.h"

#include <stdio.h>
#include <stdlib.h>

#ifdef WITH_DMALLOC
#  include <dmalloc.h>
#endif

#define BUFFER_SIZE 256

/* {{{ _digest_to_str */
static char *
_digest_to_str (char *str, unsigned char *digest)
{
  int i = 0;
  char *sum = str ? str : (char *) calloc (sizeof (char), 33);

  if (sum)
    {
      for (; i < 16; ++i, sum += 2)
	sprintf (sum, "%02x", digest[i]);
      *sum = '\0';
      sum -= 32;
    }

  return (sum);
}

/* }}} */

/* {{{ md5sum_from_fp */
char *
md5sum_from_fp (FILE * fp, char *sum)
{
  MD5 md5;
  size_t count;
  char buf[BUFFER_SIZE];
  unsigned char digest[16];

  MD5Open (&md5);

  while ((count = fread (buf, 1, BUFFER_SIZE, fp)) > 0)
    MD5Digest (&md5, buf, count);

  if (count == -1)
    return (NULL);

  MD5Close (&md5, digest);

  return (_digest_to_str (sum, digest));
}

/* }}} */

/* {{{ md5sum_from_file */
char *
md5sum_from_file (const char *filename, char *sum)
{
  FILE *fp;
  char *result;

  if (!(fp = fopen (filename, "rb")))
    return (NULL);

  result = md5sum_from_fp (fp, sum);
  fclose (fp);

  return (result);
}

/* }}} */

/*
 * Local variables:
 * tab-width: 8
 * c-basic-offset: 8
 * End:
 * vim600: noet nonu sw=8 ts=8 fdm=marker
 * vim<600: noet nonu sw=8 ts=8
 */
