/* version.c -- Version management
 * 
 * Copyright (C) 2004 H Zero Seven
 *
 * Authors: Christian Volmering <risk@h07.org>
 * 
 * This file is part of the aexpl-ng distribution.
 * 
 * The aexpl-ng distribution is free software; you can redistribute it 
 * and/or modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 of 
 * the License, or (at your option) any later version.
 *
 * The aexpl-ng distribution is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied 
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the aexpl-ng distribution; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 * MA 02111-1307 USA
 */

#include "config.h"
#include "version.h"

#include <sys/types.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#ifndef HAVE_LIBGNUREGEX
#  include <regex.h>
#else
#  include <gnuregex.h>
#endif

#ifdef WITH_DMALLOC
#  include <dmalloc.h>
#endif

/* {{{ parse_version_str */
int
parse_version_str (const char *str, struct _ae_version_t *version)
{
  regex_t buffer;
  struct re_registers regs;
  char tmp[42];
  int num, pos;
  char *regex = "([0-9]{1,2})\\.([0-9]{1,2})"
    "((a|b)([0-9]))?(\\-([a-zA-Z]{1,32}))?.*";

  re_set_syntax (RE_SYNTAX_POSIX_EXTENDED);
  memset (&buffer, 0, sizeof (buffer));

  if (re_compile_pattern (regex, strlen (regex) + 1, &buffer))
    return (0);

  buffer.regs_allocated = REGS_UNALLOCATED;
  if (re_match (&buffer, str, strlen (str) + 1, 0, &regs) < 0)
    return (0);

  for (num = 0; num < regs.num_regs; ++num)
    {
      if (*(regs.start + num) == -1)
	break;

      pos = *(regs.end + num) - *(regs.start + num);
      strncpy (tmp, &*(str + *(regs.start + num)), pos);
      *(tmp + pos) = '\0';

      switch (num)
	{
	case 1:
	  sscanf (tmp, "%2hu", &version->major);
	  break;
	case 2:
	  sscanf (tmp, "%2hu", &version->minor);
	  break;
	case 4:
	  sscanf (tmp, "%c", &version->state);
	  break;
	case 5:
	  sscanf (tmp, "%1hu", &version->release);
	  break;
	case 7:
	  sscanf (tmp, "%32s", version->suffix);
	  break;
	}
    }

  regfree (&buffer);
  return (1);
}

/* }}} */

/* {{{ get_version_str */
void
get_version_str (struct _ae_version_t *version, char *buffer)
{
  char *tmp = buffer;

  sprintf (tmp, "%d.%d", version->major, version->minor);
  tmp += 3 + (version->major > 9 ? 1 : 0) + (version->minor > 9 ? 1 : 0);

  if (version->state != 0)
    {
      sprintf (tmp, "%c%d", version->state, version->release);
      tmp += 2;
    }

  if (*(version->suffix) != 0)
    sprintf (tmp, "%s", version->suffix);
}

/* }}} */

/*
 * Local variables:
 * tab-width: 8
 * c-basic-offset: 8
 * End:
 * vim600: noet nonu sw=8 ts=8 fdm=marker
 * vim<600: noet nonu sw=8 ts=8
 */
